#! /usr/bin/env python

###########################################################
# Power Timer Board Sample Program  v 1.0
###########################################################
# To run first install dependancies :
#
# apt-get install python-smbus python-dateutil
#
############################################################
#
# Command line syntax
#
# pwrtimer.py [delay in mins default 30]
#
#
# IMPORTANT :
#
# The RTC alarm matches all of month, day, hour, min, seconds
# so you cannot re-use an alarm like an bed side alarm clock
# 
# The alarm once activated remains set unless actively unset 
#############################################################

import argparse
from smbus import SMBus
from calendar import isleap
from smbus import SMBus
from datetime import datetime, timedelta
from dateutil import tz
import os

################################################ RTC FUNCTIONS START
RTCADR = 0x6f
STBIT = 0x80
LPYR  = 0x20
almbase = [0xa, 0x11]

def tobcd(val):
  return (val % 10) | (val / 10) << 4

def writertc(i2cbus, dt):
  sec = dt.second
  min = dt.minute
  hour = dt.hour
  # rtc-ds1307 uses weekday convention Sun = 1, Sat = 7
  wkday = (dt.weekday() + 1)%7 + 1
  day = dt.day
  month = dt.month
  year = dt.year
  leap = isleap(year)

  data = [0,0,0,0,0,0,0]
  # We're adding the start bit to the seconds to enable the rtc oscillator.
  data[0] = tobcd(sec) | STBIT
  data[1] = tobcd(min)
  data[2] = tobcd(hour)
  bits345 = i2cbus.read_byte_data(RTCADR, 3) & 0x38
  data[3] = bits345 | wkday
  data[4] = tobcd(day)
  data[5] = tobcd(month)
  if leap:
    data[5] |= LPYR
  data[6] = tobcd(year % 100)

  i2cbus.write_i2c_block_data(RTCADR, 0, data)

def writealm(i2cbus, alm, dt):
  if alm > 0:
    alm = 1
  else:
    alm = 0
  sec = dt.second
  min = dt.minute
  hour = dt.hour
  # rtc-ds1307 uses weekday convention Sun = 1, Sat = 7
  # wkday in alarm has to match the wkday of rtc time for the alarm to trigger
  wkday = (dt.weekday() + 1)%7 + 1
  day = dt.day
  month = dt.month
  year = dt.year

  data = [0,0,0,0,0,0]
  #data[0] = tobcd(sec)
  # Always at zero second match, so we only give min resolution
  data[0] = tobcd(0)
  data[1] = tobcd(min)
  data[2] = tobcd(hour)
  data[3] = 0x70 | wkday  # ALM0MSK = 111
  data[4] = tobcd(day)
  data[5] = tobcd(month)

  i2cbus.write_i2c_block_data(RTCADR, almbase[alm], data)

def readrtc(i2cbus):
  data=i2cbus.read_i2c_block_data(RTCADR, 0, 7)

  sec   = (data[0] & 0x7f) / 16 * 10 + (data[0] & 0x0f)
  min   = data[1] / 16 * 10 + (data[1] & 0x0f)
  hour  = data[2] / 16 * 10 + (data[2] & 0x0f)
  day   = data[4] / 16 * 10 + (data[4] & 0x0f)
  month = (data[5] & 0x10) / 16 * 10 + (data[5] & 0x0f)
  year  = data[6] / 16 * 10 + (data[6] & 0x0f)
  dt = datetime(2000+year, month, day, hour, min, sec)
  return dt

def readalm(i2cbus, alm):
  if alm > 0:
    alm = 1
  else:
    alm = 0
  data = i2cbus.read_i2c_block_data(RTCADR, almbase[alm], 6)

  sec   = data[0] / 16 * 10 + (data[0] & 0x0f)
  min   = data[1] / 16 * 10 + (data[1] & 0x0f)
  hour  = data[2] / 16 * 10 + (data[2] & 0x0f)
  day   = data[4] / 16 * 10 + (data[4] & 0x0f)
  month = data[5] / 16 * 10 + (data[5] & 0x0f)
  # year not used in alarm time
  dt = datetime(2000, month, day, hour, min, sec)
  return dt

def enablealm0(i2cbus):
  data = i2cbus.read_byte_data(RTCADR, 7)
  data |= 0x10
  i2cbus.write_byte_data(RTCADR, 7, data)

def enablealm1(i2cbus):
  data = i2cbus.read_byte_data(RTCADR, 7)
  data |= 0x20
  i2cbus.write_byte_data(RTCADR, 7, data)

def disablealm0(i2cbus):
  # When disabling the alarm, keep the mfp output high 
  data = i2cbus.read_byte_data(RTCADR, 7)
  data &= 0xef
  data |= 0x80
  i2cbus.write_byte_data(RTCADR, 7, data)

def disablealm1(i2cbus):
  # When disabling the alarm, keep the mfp output high
  data = i2cbus.read_byte_data(RTCADR, 7)
  data &= 0xdf
  data |= 0x80
  i2cbus.write_byte_data(RTCADR, 7, data)

################################################ RTC FUNCTIONS END

parser = argparse.ArgumentParser()
parser.add_argument('delay', nargs='?', default=30, type=int, help='delay in mins till wakeup, default 30')
args=parser.parse_args()

# 7940N lives on I2C1
i2cbus = SMBus(1)

# Note that we're only using alarm0, not both
# If you want to use both alarms to trigger wakeups, then only one alarm can be enabled at a time
# The idea is that you'd have to disable alarm 0 then enable alarm 1, so not entirely useful for us
# See the datasheet section 5.5 for details.

print "Disabling Alarms"
disablealm0(i2cbus)
disablealm1(i2cbus)

# As this isn't the main RTC we need to update it to ensure it's correct
print "Writing current time to RTC"
writertc(i2cbus, datetime.utcnow())

dtrtc = readrtc(i2cbus)

rtctime = dtrtc.strftime('%d-%b %H:%M:%S')
print "RTC Time Now Set To  : ",rtctime

delta=timedelta(minutes = args.delay)


print "Setting Alarm Time"
wakeup = dtrtc + delta
writealm(i2cbus, 0, wakeup)

alm0time = readalm(i2cbus, 0).strftime('%d-%b %H:%M:%S')
print "Alarm Time Set To    : ", alm0time

print "Enabling Alarm"
enablealm0(i2cbus)
